"use strict";
exports.__esModule = true;
var core_1 = require("@babel/core");
var helper_module_imports_1 = require("@babel/helper-module-imports");
function functionDeclarationToExpression(declaration) {
    return core_1.types.functionExpression(declaration.id, declaration.params, declaration.body, declaration.generator, declaration.async);
}
function classDeclarationToExpression(declaration) {
    return core_1.types.classExpression(declaration.id, declaration.superClass, declaration.body, declaration.decorators);
}
function getFileName(state) {
    var filename = state.filename, cwd = state.cwd;
    if (!filename) {
        return undefined;
    }
    if (cwd && filename.startsWith(cwd)) {
        return filename.slice(cwd.length);
    }
    return filename;
}
var functionsToReplace = ['getServerSideProps', 'getStaticProps'];
function transformPropGetters(path, transform) {
    var node = path.node;
    if (core_1.types.isFunctionDeclaration(node.declaration)) {
        var functionId = node.declaration.id;
        if (!functionId) {
            return;
        }
        if (!functionsToReplace.includes(functionId.name)) {
            return;
        }
        node.declaration = core_1.types.variableDeclaration('const', [
            core_1.types.variableDeclarator(functionId, transform(functionDeclarationToExpression(node.declaration))),
        ]);
        return;
    }
    if (core_1.types.isVariableDeclaration(node.declaration)) {
        node.declaration.declarations.forEach(function (declaration) {
            if (core_1.types.isIdentifier(declaration.id) &&
                functionsToReplace.includes(declaration.id.name) &&
                declaration.init) {
                declaration.init = transform(declaration.init);
            }
        });
    }
}
function addWithSuperJSONPropsImport(path) {
    return helper_module_imports_1.addNamed(path, 'withSuperJSONProps', 'babel-plugin-superjson-next/tools');
}
function addWithSuperJSONPageImport(path) {
    return helper_module_imports_1.addNamed(path, 'withSuperJSONPage', 'babel-plugin-superjson-next/tools');
}
function superJsonWithNext() {
    return {
        name: 'replace gSSP',
        visitor: {
            Program: function (path, state) {
                var _a;
                var filename = (_a = getFileName(state)) !== null && _a !== void 0 ? _a : 'pages/Default.js';
                if (!filename.includes('pages/')) {
                    return;
                }
                var foundGSSP = false;
                path.traverse({
                    ExportNamedDeclaration: function (path) {
                        transformPropGetters(path, function (decl) {
                            foundGSSP = true;
                            return core_1.types.callExpression(addWithSuperJSONPropsImport(path), [
                                decl,
                            ]);
                        });
                    }
                });
                if (foundGSSP) {
                    path.traverse({
                        ExportDefaultDeclaration: function (path) {
                            function wrapInHOC(expr) {
                                return core_1.types.callExpression(addWithSuperJSONPageImport(path), [
                                    expr,
                                ]);
                            }
                            var node = path.node;
                            if (core_1.types.isIdentifier(node.declaration)) {
                                node.declaration = wrapInHOC(node.declaration);
                            }
                            if (core_1.types.isFunctionExpression(node.declaration)) {
                                node.declaration = wrapInHOC(node.declaration);
                            }
                            if (core_1.types.isFunctionDeclaration(node.declaration) ||
                                core_1.types.isClassDeclaration(node.declaration)) {
                                if (node.declaration.id) {
                                    path.insertBefore(node.declaration);
                                    node.declaration = wrapInHOC(node.declaration.id);
                                }
                                else {
                                    if (core_1.types.isFunctionDeclaration(node.declaration)) {
                                        node.declaration = wrapInHOC(functionDeclarationToExpression(node.declaration));
                                    }
                                    else {
                                        node.declaration = wrapInHOC(classDeclarationToExpression(node.declaration));
                                    }
                                }
                            }
                        }
                    });
                }
            }
        }
    };
}
exports["default"] = superJsonWithNext;
//# sourceMappingURL=index.js.map